Package: GMCM
Type: Package
Title: Fast Estimation of Gaussian Mixture Copula Models
Description: Unsupervised Clustering and Meta-analysis using Gaussian Mixture
    Copula Models.
Version: 1.4
Authors@R: c(
    person(given = c("Anders", "Ellern"),
           family = "Bilgrau",
           role = c("aut", "cre", "cph"),
           email = "anders.ellern.bilgrau@gmail.com", 
           comment = c(ORCID = "0000-0001-9875-2902")),
    person(given = c("Poul", "Svante"),
           family = "Eriksen",
           role = c("ths", "ctb"),
           comment = c(ORCID = "0000-0001-9192-1814")),
    person(given = "Martin",
           family = "Boegsted",
           role = c("ths", "ctb"))
    )
Maintainer: Anders Ellern Bilgrau <anders.ellern.bilgrau@gmail.com>
URL: https://github.com/AEBilgrau/GMCM
BugReports: https://github.com/AEBilgrau/GMCM/issues
License: GPL (>= 2)
KeepSource: yes
Imports: Rcpp (>= 0.10.6), ellipse
LinkingTo: Rcpp, RcppArmadillo
Suggests: idr, Hmisc, RColorBrewer, foreach, jpeg, testthat (>= 0.3),
        knitr, rmarkdown, shiny, shinydashboard, shinyBS,
        rhandsontable, DT
VignetteBuilder: knitr
RoxygenNote: 6.1.1
Encoding: UTF-8
NeedsCompilation: yes
Packaged: 2019-11-05 18:13:37 UTC; anders
Author: Anders Ellern Bilgrau [aut, cre, cph]
    (<https://orcid.org/0000-0001-9875-2902>),
  Poul Svante Eriksen [ths, ctb]
    (<https://orcid.org/0000-0001-9192-1814>),
  Martin Boegsted [ths, ctb]
Repository: CRAN
Date/Publication: 2019-11-05 22:10:02 UTC
Built: R 4.6.0; aarch64-apple-darwin23; 2026-04-25 19:19:05 UTC; unix
Archs: GMCM.so.dSYM
