# Generated by roxygen2: do not edit by hand

S3method(plot,theta)
S3method(print,theta)
S3method(summary,theta)
export(EMAlgorithm)
export(SimulateGMCMData)
export(SimulateGMMData)
export(Uhat)
export(as.theta)
export(choose.theta)
export(classify)
export(dmvnormal)
export(fit.full.GMCM)
export(fit.general.GMCM)
export(fit.meta.GMCM)
export(fit.special.GMCM)
export(full2meta)
export(get.IDR)
export(get.prob)
export(goodness.of.fit)
export(is.theta)
export(meta2full)
export(rmvnormal)
export(rtheta)
export(runGMCM)
importFrom(Rcpp,evalCpp)
importFrom(ellipse,ellipse)
importFrom(graphics,contour)
importFrom(graphics,lines)
importFrom(graphics,points)
importFrom(stats,approxfun)
importFrom(stats,cov)
importFrom(stats,cov.wt)
importFrom(stats,cov2cor)
importFrom(stats,kmeans)
importFrom(stats,optim)
importFrom(stats,qnorm)
importFrom(stats,rchisq)
importFrom(stats,rnorm)
importFrom(stats,runif)
importFrom(utils,flush.console)
importFrom(utils,write.csv2)
useDynLib(GMCM)
