---
title: "Bug reports"
author: "Anders Ellern Bilgrau"
date: "2019-03-25"
output: html_document
---



## Bug reports

If you encounter a bug or issue with the web-application or package itself, please submit an issue at [the GitHub issues page](https://github.com/AEBilgrau/GMCM/issues). 
Requests for features are also warmly welcome here.
If possible try to describe the issue so it can be reproduced.

Alternatively, if you do not have a GitHub account, feel free to email me at anders.ellern.bilgrau (at) gmail.com with your problems or suggestions.


Please refer to **GMCM version 1.3.1** when filing a report.

---

<div align="center">
  Anders Ellern Bilgrau
  &mdash;
  <a href="https://github.com/AEBilgrau/GMCM/labels/shiny">
    <i class="fa fa-bug"></i>
  </a>
  &mdash;
  2019
</div>
