---
title: "Bug reports"
author: "Anders Ellern Bilgrau"
date: "`r Sys.Date()`"
output: html_document
---

```{r setup, include=FALSE}
knitr::opts_chunk$set(echo = TRUE)
library("htmltools")
```

## Bug reports

If you encounter a bug or issue with the web-application or package itself,
please submit an issue at [the GitHub issues
page](https://github.com/AEBilgrau/GMCM/issues). Requests for features are also
warmly welcome. If possible try to describe the issue so it can be reproduced.
You can for example also include the source generated via the report button.

If you do not have a GitHub account, feel free to email me at
anders.ellern.bilgrau (at) gmail.com with your problems or suggestions.
Please refer to **GMCM version `r packageVersion("GMCM")`** when filing a
report.

---

```{r bug, echo = FALSE, results='asis'}
cat(
as.character(
  shiny::withTags(
    div(
      align = "center", 
      "Anders Ellern Bilgrau",
      HTML("&mdash;"),
      a(href = "https://github.com/AEBilgrau/GMCM/labels/shiny",
        shiny::icon("bug")),
      HTML("&mdash;"),
      "2019"
    )
  )
))
```
